import asyncio
import logging
from datetime import date, datetime, timedelta
from itertools import chain
from typing import ClassVar, Literal, overload
from zoneinfo import ZoneInfo

import aiohttp
import boto3  # type: ignore
from aiohttp.client_exceptions import ClientResponseError

from .aws_srp import AWSSRP
from .exceptions import HydroOttawaConnectionError, HydroOttawaInvalidAuth
from .models import (
    AggregateType,
    BillingPeriod,
    DailyRead,
    HourlyRead,
    HourlyResponse,
    MonthlyRead,
)

logger = logging.getLogger(__name__)
DEBUG_LOG_RESPONSE = False


class HydroOttawa:
    """Class that can get historical and forecasted usage/cost from Bidgely's NA API."""

    BASE_URL: ClassVar[str] = "https://api-myaccount.hydroottawa.com"

    def __init__(
        self,
        session: aiohttp.ClientSession,
        username: str,
        password: str,
    ):
        self.session: aiohttp.ClientSession = session
        self.username: str = username
        self.password: str = password
        self.x_access: str | None = None
        self.x_id: str | None = None
        self.auth_token: str | None = None

    async def async_login(self) -> None:
        "Returns user-id and token for Bidgely."
        client = await asyncio.get_running_loop().run_in_executor(
            None,
            boto3.client,
            "cognito-idp",
            "ca-central-1",
        )  # type: ignore
        aws = AWSSRP(
            username=self.username,
            password=self.password,
            pool_id="ca-central-1_VYnwOhMBK",
            client_id="7scfcis6ecucktmp4aqi1jk6cb",
            client=client,
            loop=self.session.loop,
        )
        try:
            tokens = await aws.authenticate_user()
            self.x_access = tokens["AuthenticationResult"]["AccessToken"]
            self.x_id = tokens["AuthenticationResult"]["IdToken"]

            headers = {"x-id": self.x_id, "x-access": self.x_access}

            async with self.session.get(
                f"{self.BASE_URL}/app-token", headers=headers
            ) as resp:
                self.auth_token = resp.headers["x-amzn-remapped-authorization"]
        except ClientResponseError as err:
            if err.status in (401, 403):
                raise HydroOttawaInvalidAuth() from err
            else:
                raise HydroOttawaConnectionError() from err
        return None

    async def fetch_hourly(self, fetch_date: date | None = None) -> list[HourlyRead]:
        if fetch_date is None:
            fetch_date = datetime.now(ZoneInfo("America/Toronto")) - timedelta(days=1)
            fetch_date = fetch_date.date()
        url = f"{self.BASE_URL}/usage/consumption/hourly"
        headers = {
            "x-id": self.x_id,
            "x-access": self.x_access,
            "Authorization": self.auth_token,
        }
        async with self.session.post(
            url, headers=headers, json={"date": f"{fetch_date}"}
        ) as resp:
            reads = await resp.json()
            processed = [HourlyRead.from_dict(read) for read in reads["intervals"]]
            # su = HourlySummary.from_dict(reads["summary"])
            total = HourlyResponse.from_dict(reads)
            processed = total.measurements
        return processed

    async def fetch_daily(
        self, start_date: date | None = None, end_date: date | None = None
    ) -> list[DailyRead]:
        if start_date is None:
            start_date = datetime.now(ZoneInfo("America/Toronto")) - timedelta(days=8)
            start_date = start_date.date()
        if end_date is None:
            end_date = datetime.now(ZoneInfo("America/Toronto")).date() - timedelta(
                days=1
            )
        url = f"{self.BASE_URL}/usage/consumption/daily"
        headers = {
            "x-id": self.x_id,
            "x-access": self.x_access,
            "Authorization": self.auth_token,
        }
        async with self.session.post(
            url,
            headers=headers,
            json={"startDate": f"{start_date}", "endDate": f"{end_date}"},
        ) as resp:
            processed: list[DailyRead] = []
            reads = await resp.json()
            processed = [DailyRead.from_dict(read) for read in reads["intervals"]]
        return processed

    async def fetch_monthly(
        self, start_date: date | None = None, end_date: date | None = None
    ) -> list[MonthlyRead]:
        if start_date is None:
            start_date = datetime.now(ZoneInfo("America/Toronto")) - timedelta(days=365)
            start_date = start_date.date()
        if end_date is None:
            end_date = datetime.now(ZoneInfo("America/Toronto")).date() - timedelta(
                days=1
            )
        url = f"{self.BASE_URL}/usage/consumption/monthly"
        headers = {
            "x-id": self.x_id,
            "x-access": self.x_access,
            "Authorization": self.auth_token,
        }
        async with self.session.post(
            url,
            headers=headers,
            json={"startDate": f"{start_date}", "endDate": f"{end_date}"},
        ) as resp:
            reads = await resp.json()
            processed = [MonthlyRead.from_dict(read) for read in reads["intervals"]]
        return processed

    async def get_billing_periods(self) -> list[BillingPeriod]:
        headers = {
            "x-id": self.x_id,
            "x-access": self.x_access,
            "Authorization": self.auth_token,
        }
        async with self.session.get(
            f"{self.BASE_URL}/usage/billing-period-list", headers=headers
        ) as resp:
            reads = await resp.json()
            billing_periods = [BillingPeriod.from_dict(period) for period in reads]
        return billing_periods

    @overload
    async def get_usage(
        self,
        start_date: date | None,
        end_date: date | None,
        aggregate: Literal[AggregateType.HOURLY],
    ) -> list[HourlyRead]: ...

    @overload
    async def get_usage(
        self,
        start_date: date | None,
        end_date: date | None,
        aggregate: Literal[AggregateType.DAILY],
    ) -> list[DailyRead]: ...

    @overload
    async def get_usage(
        self,
        start_date: date | None,
        end_date: date | None,
        aggregate: Literal[AggregateType.MONTHLY],
    ) -> list[MonthlyRead]: ...

    async def get_usage(
        self,
        start_date: date | None = None,
        end_date: date | None = None,
        aggregate: AggregateType = AggregateType.HOURLY,
    ):
        if start_date is None:
            start_date = datetime.now(ZoneInfo("America/Toronto")) - timedelta(days=8)
            start_date = start_date.date()
        if end_date is None:
            end_date = datetime.now(ZoneInfo("America/Toronto")).date() - timedelta(
                days=1
            )

        match aggregate:
            case AggregateType.HOURLY:
                tasks = []
                days = (
                    start_date + timedelta(days=x)
                    for x in range((end_date - start_date).days + 1)
                )
                for day in days:
                    single_day = self.fetch_hourly(day)
                    tasks.append(single_day)
                result = await asyncio.gather(*tasks)
                hourlies: list[HourlyRead] = list(chain.from_iterable(result))
                return hourlies
            case AggregateType.DAILY:
                return await self.fetch_daily(start_date, end_date)
            case AggregateType.MONTHLY:
                logger.error("Monthly readings are not supported yet.")
                monthlies: list[MonthlyRead] = []
                return monthlies



import logging

from .hydro_ottawa import HydroOttawa
from .models import AggregateType, BillRead, DailyRead, HourlyRead, MonthlyRead

__all__ = [
    "AggregateType",
    "DailyRead",
    "HourlyRead",
    "HydroOttawa",
    "BillUsage",
    "BillRead",
    "MonthlyRead",
]

logging.getLogger("hydro_ottawa").addHandler(logging.NullHandler())



from dataclasses import dataclass, field
from datetime import date, datetime
from enum import StrEnum, auto
from zoneinfo import ZoneInfo

from mashumaro import field_options
from mashumaro.mixins.json import DataClassDictMixin


class RateBands(StrEnum):
    TIER_1 = auto()
    TIER_2 = auto()
    TIERED = "TIERED"


class AggregateType(StrEnum):
    HOURLY = auto()
    DAILY = auto()
    MONTHLY = auto()


@dataclass(slots=True)
class HourlyRead(DataClassDictMixin):
    start_time: datetime = field(
        metadata=field_options(
            alias="startDateTime",
            deserialize=lambda d: datetime.fromisoformat(d).replace(
                tzinfo=ZoneInfo("America/Toronto")
            ),
        )
    )
    end_time: datetime = field(
        metadata=field_options(
            alias="endDateTime",
            deserialize=lambda d: datetime.fromisoformat(d).replace(
                tzinfo=ZoneInfo("America/Toronto")
            ),
        )
    )
    rate_band: str = field(metadata=field_options(alias="rateBand"))
    usage: float = field(metadata=field_options(alias="hourlyUsage"))
    cost: float = field(metadata=field_options(alias="hourlyCost"))


@dataclass(slots=True)
class HourlySummary(DataClassDictMixin):
    account_id: str = field(metadata=field_options(alias="accountId"))
    actual_date: date = field(metadata=field_options(alias="actualDate"))
    rate_plan: str = field(metadata=field_options(alias="ratePlan"))
    billing_period_start_date: datetime = field(
        metadata=field_options(alias="billingPeriodStartDate")
    )
    billing_period_end_date: datetime = field(
        metadata=field_options(alias="billingPeriodEndDate")
    )
    total_usage: float = field(metadata=field_options(alias="totalUsage"))
    total_cost: float = field(metadata=field_options(alias="totalCost"))
    hourly_average_usage: float = field(
        metadata=field_options(alias="hourlyAverageUsage")
    )
    hourly_average_cost: float = field(
        metadata=field_options(alias="hourlyAverageCost")
    )
    total_off_peak_usage: float = field(
        metadata=field_options(alias="totalOffPeakUsage")
    )
    total_off_peak_cost: float = field(metadata=field_options(alias="totalOffPeakCost"))
    total_mid_peak_usage: float = field(
        metadata=field_options(alias="totalMidPeakUsage")
    )
    total_mid_peak_cost: float = field(metadata=field_options(alias="totalMidPeakCost"))
    total_on_peak_usage: float = field(metadata=field_options(alias="totalOnPeakUsage"))
    total_on_peak_cost: float = field(metadata=field_options(alias="totalOnPeakCost"))
    total_ulo_peak_usage: float = field(metadata=field_options(alias="totalUloUsage"))
    total_ulo_peak_cost: float = field(metadata=field_options(alias="totalUloCost"))
    number_of_hours: int = field(metadata=field_options(alias="numberOfHours"))


@dataclass(slots=True)
class HourlyResponse(DataClassDictMixin):
    summary: HourlySummary = field(metadata=field_options(alias="summary"))
    measurements: list[HourlyRead] = field(metadata=field_options(alias="intervals"))


@dataclass(slots=True)
class DailyRead(DataClassDictMixin):
    measurement_time: date = field(metadata=field_options(alias="date"))
    usage: float = field(metadata=field_options(alias="dailyUsage"))


@dataclass(slots=True)
class MonthlyRead(DataClassDictMixin):
    start_date: date = field(metadata=field_options(alias="startDate"))
    end_date: date = field(metadata=field_options(alias="startDate"))
    monthly_usage: int = field(metadata=field_options(alias="monthlyUsage"))
    off_peak_usage: int = field(metadata=field_options(alias="offPeakUsage"))
    mid_peak_usage: int = field(metadata=field_options(alias="midPeakUsage"))
    on_peak_usage: int = field(metadata=field_options(alias="onPeakUsage"))
    ulo_usage: int = field(metadata=field_options(alias="uloUsage"))
    monthly_cost: float = field(metadata=field_options(alias="monthlyCost"))
    off_peak_cost: float = field(metadata=field_options(alias="offPeakCost"))
    mid_peak_cost: float = field(metadata=field_options(alias="midPeakUsage"))
    on_peak_cost: float = field(metadata=field_options(alias="onPeakCost"))
    ulo_cost: float = field(metadata=field_options(alias="uloCost"))
    rate_plan: str = field(metadata=field_options(alias="ratePlan"))


@dataclass(slots=True)
class BillingPeriod(DataClassDictMixin):
    start_date: date = field(metadata=field_options(alias="startDate"))
    end_date: date = field(metadata=field_options(alias="endDate"))
    rate_plan: RateBands = field(metadata=field_options(alias="ratePlan"))
    current_billing_period: bool = field(
        metadata=field_options(alias="currentBillingPeriod")
    )


@dataclass(slots=True)
class BillRead(DataClassDictMixin):
    start_date: date



"""AWS SRP"""

# https://github.com/dotKrad/hass-fpl/blob/master/custom_components/fpl/aws_srp.py
from __future__ import annotations

import asyncio
import base64
import binascii
import datetime
import functools
import hashlib
import hmac
import os
import re
from typing import TYPE_CHECKING, Any, Final

import boto3

if TYPE_CHECKING:
    from mypy_boto3_cognito_idp.client import CognitoIdentityProviderClient
    from mypy_boto3_cognito_idp.type_defs import RespondToAuthChallengeResponseTypeDef

# https://github.com/aws/amazon-cognito-identity-js/blob/master/src/AuthenticationHelper.js#L22
n_hex = (
    "FFFFFFFFFFFFFFFFC90FDAA22168C234C4C6628B80DC1CD1"
    + "29024E088A67CC74020BBEA63B139B22514A08798E3404DD"
    + "EF9519B3CD3A431B302B0A6DF25F14374FE1356D6D51C245"
    + "E485B576625E7EC6F44C42E9A637ED6B0BFF5CB6F406B7ED"
    + "EE386BFB5A899FA5AE9F24117C4B1FE649286651ECE45B3D"
    + "C2007CB8A163BF0598DA48361C55D39A69163FA8FD24CF5F"
    + "83655D23DCA3AD961C62F356208552BB9ED529077096966D"
    + "670C354E4ABC9804F1746C08CA18217C32905E462E36CE3B"
    + "E39E772C180E86039B2783A2EC07A28FB5C55DF06F4C52C9"
    + "DE2BCBF6955817183995497CEA956AE515D2261898FA0510"
    + "15728E5A8AAAC42DAD33170D04507A33A85521ABDF1CBA64"
    + "ECFB850458DBEF0A8AEA71575D060C7DB3970F85A6E1E4C7"
    + "ABF5AE8CDB0933D71E8C94E04A25619DCEE3D2261AD2EE6B"
    + "F12FFA06D98A0864D87602733EC86A64521F2B18177B200C"
    + "BBE117577A615D6C770988C0BAD946E208E24FA074E5AB31"
    + "43DB5BFCE0FD108E4B82D120A93AD2CAFFFFFFFFFFFFFFFF"
)
# https://github.com/aws/amazon-cognito-identity-js/blob/master/src/AuthenticationHelper.js#L49
G_HEX: Final[str] = "2"
INFO_BITS: Final[bytearray] = bytearray("Caldera Derived Key", "utf-8")


def hash_sha256(buf: bytes) -> str:
    """AuthenticationHelper.hash"""
    a = hashlib.sha256(buf).hexdigest()
    return (64 - len(a)) * "0" + a


def hex_hash(hex_string: str) -> str:
    return hash_sha256(bytearray.fromhex(hex_string))


def hex_to_long(hex_string: str | bytes) -> int:
    return int(hex_string, 16)


def long_to_hex(long_num: int) -> str:
    return "%x" % long_num


def get_random(nbytes: int) -> int:
    random_hex = binascii.hexlify(os.urandom(nbytes))
    return hex_to_long(random_hex)


def pad_hex(long_int: int | str) -> str:
    """
    Converts a Long integer (or hex string) to hex format padded with zeroes for hashing
    :param {Long integer|String} long_int Number or string to pad.
    :return {String} Padded hex string.
    """
    if not isinstance(long_int, str):
        hash_str = long_to_hex(long_int)
    else:
        hash_str = long_int
    if len(hash_str) % 2 == 1:
        hash_str = "0%s" % hash_str
    elif hash_str[0] in "89ABCDEFabcdef":
        hash_str = "00%s" % hash_str
    return hash_str


def compute_hkdf(ikm: bytes, salt: bytes) -> bytes:
    """
    Standard hkdf algorithm
    :param {Buffer} ikm Input key material.
    :param {Buffer} salt Salt value.
    :return {Buffer} Strong key material.
    @private
    """
    prk = hmac.new(salt, ikm, hashlib.sha256).digest()
    info_bits_update = INFO_BITS + bytearray(chr(1), "utf-8")
    hmac_hash = hmac.new(prk, info_bits_update, hashlib.sha256).digest()
    return hmac_hash[:16]


def calculate_u(big_a: int, big_b: int) -> int:
    """
    Calculate the client's value U which is the hash of A and B
    :param {Long integer} big_a Large A value.
    :param {Long integer} big_b Server B value.
    :return {Long integer} Computed U value.
    """
    u_hex_hash = hex_hash(pad_hex(big_a) + pad_hex(big_b))
    return hex_to_long(u_hex_hash)


class ForceChangePasswordException(Exception):
    """Raised when the user is forced to change their password"""


class AWSSRP(object):
    NEW_PASSWORD_REQUIRED_CHALLENGE: Final = "NEW_PASSWORD_REQUIRED"
    PASSWORD_VERIFIER_CHALLENGE: Final = "PASSWORD_VERIFIER"

    def __init__(
        self,
        username: str,
        password: str,
        pool_id: str,
        client_id: str,
        loop: asyncio.AbstractEventLoop,
        pool_region: str | None = None,
        client: CognitoIdentityProviderClient | None = None,
        client_secret: str | None = None,
    ):
        if pool_region is not None and client is not None:
            raise ValueError(
                "pool_region and client should not both be specified "
                "(region should be passed to the boto3 client instead)"
            )

        self.username = username
        self.password = password
        self.pool_id = pool_id
        self.client_id = client_id
        self.client_secret = client_secret
        self.client = (
            client if client else boto3.client("cognito-idp", region_name=pool_region)
        )
        self.big_n = hex_to_long(n_hex)
        self.g = hex_to_long(G_HEX)
        self.k = hex_to_long(hex_hash("00" + n_hex + "0" + G_HEX))
        self.small_a_value = self.generate_random_small_a()
        self.large_a_value = self.calculate_a()
        self.loop = loop

    def generate_random_small_a(self) -> int:
        """
        helper function to generate a random big integer
        :return {Long integer} a random value.
        """
        random_long_int = get_random(128)
        return random_long_int % self.big_n

    def calculate_a(self) -> int:
        """
        Calculate the client's public value A = g^a%N
        with the generated random number a
        :param {Long integer} a Randomly generated small A.
        :return {Long integer} Computed large A.
        """
        big_a = pow(self.g, self.small_a_value, self.big_n)
        # safety check
        if (big_a % self.big_n) == 0:
            raise ValueError("Safety check for A failed")
        return big_a

    def get_password_authentication_key(
        self, username: str, password: str, server_b_value: int, salt: int
    ) -> bytes:
        """
        Calculates final hkdf based on computed S value, computed U value, and the key
        :param {String} username Username.
        :param {String} password Password.
        :param {Long integer} server_b_value Server B value.
        :param {Long integer} salt Generated salt.
        :return {Buffer} Computed HKDF value.
        """
        u_value = calculate_u(self.large_a_value, server_b_value)
        if u_value == 0:
            raise ValueError("U cannot be zero.")
        username_password = "%s%s:%s" % (self.pool_id.split("_")[1], username, password)
        username_password_hash = hash_sha256(username_password.encode("utf-8"))

        x_value = hex_to_long(hex_hash(pad_hex(salt) + username_password_hash))
        g_mod_pow_xn = pow(self.g, x_value, self.big_n)
        int_value2 = server_b_value - self.k * g_mod_pow_xn
        s_value = pow(int_value2, self.small_a_value + u_value * x_value, self.big_n)
        hkdf = compute_hkdf(
            bytearray.fromhex(pad_hex(s_value)),
            bytearray.fromhex(pad_hex(long_to_hex(u_value))),
        )
        return hkdf

    def get_auth_params(self) -> dict[str, str]:
        auth_params = {
            "USERNAME": self.username,
            "SRP_A": long_to_hex(self.large_a_value),
        }
        if self.client_secret is not None:
            auth_params.update(
                {
                    "SECRET_HASH": self.get_secret_hash(
                        self.username, self.client_id, self.client_secret
                    )
                }
            )
        return auth_params

    @staticmethod
    def get_secret_hash(username: str, client_id: str, client_secret: str) -> str:
        message = bytearray(username + client_id, "utf-8")
        hmac_obj = hmac.new(bytearray(client_secret, "utf-8"), message, hashlib.sha256)
        return base64.standard_b64encode(hmac_obj.digest()).decode("utf-8")

    def process_challenge(self, challenge_parameters: dict[str, Any]) -> dict[str, Any]:
        user_id_for_srp = challenge_parameters["USER_ID_FOR_SRP"]
        salt_hex = challenge_parameters["SALT"]
        srp_b_hex = challenge_parameters["SRP_B"]
        secret_block_b64 = challenge_parameters["SECRET_BLOCK"]
        # re strips leading zero from a day number (required by AWS Cognito)
        timestamp = re.sub(
            r" 0(\d) ",
            r" \1 ",
            datetime.datetime.utcnow().strftime("%a %b %d %H:%M:%S UTC %Y"),
        )
        hkdf = self.get_password_authentication_key(
            user_id_for_srp, self.password, hex_to_long(srp_b_hex), salt_hex
        )
        secret_block_bytes = base64.standard_b64decode(secret_block_b64)
        msg = (
            bytearray(self.pool_id.split("_")[1], "utf-8")
            + bytearray(user_id_for_srp, "utf-8")
            + bytearray(secret_block_bytes)
            + bytearray(timestamp, "utf-8")
        )
        hmac_obj = hmac.new(hkdf, msg, digestmod=hashlib.sha256)
        signature_string = base64.standard_b64encode(hmac_obj.digest())
        response = {
            "TIMESTAMP": timestamp,
            "USERNAME": user_id_for_srp,
            "PASSWORD_CLAIM_SECRET_BLOCK": secret_block_b64,
            "PASSWORD_CLAIM_SIGNATURE": signature_string.decode("utf-8"),
        }
        if self.client_secret is not None:
            response.update(
                {
                    "SECRET_HASH": self.get_secret_hash(
                        self.username, self.client_id, self.client_secret
                    )
                }
            )
        return response

    async def authenticate_user(
        self, client: Any | None = None
    ) -> RespondToAuthChallengeResponseTypeDef:
        """authenticate user"""
        if client is None:
            boto_client = self.client
        else:
            boto_client = client
        auth_params = self.get_auth_params()

        response = await self.loop.run_in_executor(
            None,
            functools.partial(
                boto_client.initiate_auth,
                AuthFlow="USER_SRP_AUTH",
                AuthParameters=auth_params,
                ClientId=self.client_id,
            ),
        )

        if response["ChallengeName"] == self.PASSWORD_VERIFIER_CHALLENGE:
            challenge_response = self.process_challenge(response["ChallengeParameters"])
            tokens = await self.loop.run_in_executor(
                None,
                functools.partial(
                    boto_client.respond_to_auth_challenge,
                    ClientId=self.client_id,
                    ChallengeName=self.PASSWORD_VERIFIER_CHALLENGE,
                    ChallengeResponses=challenge_response,
                ),
            )

            if tokens.get("ChallengeName") == self.NEW_PASSWORD_REQUIRED_CHALLENGE:
                raise ForceChangePasswordException(
                    "Change password before authenticating"
                )

            return tokens
        else:
            raise NotImplementedError(
                "The %s challenge is not supported" % response["ChallengeName"]
            )

    def set_new_password_challenge(
        self, new_password: str, client: CognitoIdentityProviderClient | None = None
    ) -> RespondToAuthChallengeResponseTypeDef:  # dict[str, Any]:
        if client is None:
            boto_client = self.client
        else:
            boto_client = client
        auth_params = self.get_auth_params()
        response = boto_client.initiate_auth(
            AuthFlow="USER_SRP_AUTH",
            AuthParameters=auth_params,
            ClientId=self.client_id,
        )
        if response["ChallengeName"] == self.PASSWORD_VERIFIER_CHALLENGE:
            challenge_response = self.process_challenge(response["ChallengeParameters"])
            tokens = boto_client.respond_to_auth_challenge(
                ClientId=self.client_id,
                ChallengeName=self.PASSWORD_VERIFIER_CHALLENGE,
                ChallengeResponses=challenge_response,
            )

            if tokens["ChallengeName"] == self.NEW_PASSWORD_REQUIRED_CHALLENGE:
                challenge_response = {
                    "USERNAME": auth_params["USERNAME"],
                    "NEW_PASSWORD": new_password,
                }
                new_password_response = boto_client.respond_to_auth_challenge(
                    ClientId=self.client_id,
                    ChallengeName=self.NEW_PASSWORD_REQUIRED_CHALLENGE,
                    Session=tokens["Session"],
                    ChallengeResponses=challenge_response,
                )
                return new_password_response
            return tokens
        else:
            raise NotImplementedError(
                "The %s challenge is not supported" % response["ChallengeName"]
            )



"""Exceptions."""


class HydroOttawaError(Exception):
    """Generic Hydro Ottawa Exception."""


class HydroOttawaConnectionError(HydroOttawaError):
    """Error to indicate we cannot connect."""


class HydroOttawaInvalidAuth(HydroOttawaError):
    """Error to indicate there is invalid auth."""
